/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include "Driver\DrvUART.h"
#include "Driver\DrvUSB.h"
#include "USB\HIDSys.h"

/* Define the vendor id and product id */
#define USB_VID			0x0416
#define USB_PID			0xC140

#define HID_MOUSE       2
#define HID_KEYBOARD    1

/* Define the HID_function as either HID_MOUSE or HID_KEYBOARD */
#define HID_FUNCTION    HID_MOUSE


#define HID_DEFAULT_INT_IN_INTERVAL	20
#define HID_IS_SELF_POWERED         0
#define HID_IS_REMOTE_WAKEUP        0
#define HID_MAX_POWER               50  /* The unit is in 2mA. ex: 50 * 2mA = 100mA */

#if(HID_FUNCTION == HID_MOUSE)
# define HID_REPORT_SIZE		    3       /* The unit is in bytes */
#else
# define HID_REPORT_SIZE		    8       /* The unit is in bytes */
#endif


#define LEN_CONFIG_AND_SUBORDINATE (LEN_CONFIG+LEN_INTERFACE+LEN_HID+LEN_ENDPOINT)

/* Define the interrupt In EP number */
#define INT_IN_EP_NUM	0x01

__align(4) const S_DRVUSB_STRING_DESC
g_sVendorStringDesc =
{
	16,
	DESC_STRING,
	{'N', 'U', 'V', 'O', 'T', 'O', 'N'}
};

__align(4) const S_DRVUSB_STRING_DESC
g_sProductStringDesc =
{
	22,
	DESC_STRING,
	{
		'A', 'R', 'M', ' ',
		's', 'e', 'r', 'i', 'e', 's'
	}
};

#if 1 /* relative movement */
__align(4) const uint8_t g_HID_au8MouseReportDescriptor[] =
{
	0x05, 0x01,
	0x09, 0x02,
	0xA1, 0x01,
	0x09, 0x01,
	0xA1, 0x00,
	0x05, 0x09,
	0x19, 0x01,
	0x29, 0x03,
	0x15, 0x00,
	0x25, 0x01,
	0x95, 0x03,
	0x75, 0x01,
	0x81, 0x02,
	0x95, 0x01,
	0x75, 0x05,
	0x81, 0x01,
	0x05, 0x01,
	0x09, 0x30,
	0x09, 0x31,
	0x15, 0x81,	//ligical minimun -127	
	0x25, 0x7F,	//logical maximum 127
	0x75, 0x08,
	0x95, 0x02,
	0x81, 0x06,
	0xC0,
	0xC0
};
#else
    /* absolute movement */
__align(4) const uint8_t g_HID_au8MouseReportDescriptor[] = {
      0x05, 0x01,
      0x09, 0x02,
      0xA1, 0x01,
      0x09, 0x01,
      0xA1, 0x00,
      0x05, 0x09,
      0x19, 0x01,
      0x29, 0x03,
      0x15, 0x00,
      0x25, 0x01,
      0x95, 0x03,
      0x75, 0x01,
      0x81, 0x02,
      0x95, 0x01,
      0x75, 0x05,
      0x81, 0x01,
      0x05, 0x01,
      0x09, 0x30,
      0x09, 0x31,
      0x15, 0x81,             //logical min(-127)
      0x25, 0x7F,             //logical max(127)
      0x35, 0x00,             //physical min(0)
      0x46, 0xFF,0x00,        //physical max
      0x75, 0x08,             //report size (8 bits each)
      0x95, 0x02,             //report count (2 reports)
      0x81, 0x02,
      0xC0,
      0xC0      
};
#endif


//keyboard 101keys
__align(4) const uint8_t g_HID_au8KeyboardReportDescriptor[] = {

      0x05, 0x01,
      0x09, 0x06,
      0xA1, 0x01,
      0x05, 0x07,
      0x19, 0xE0,
      0x29, 0xE7,
      0x15, 0x00,
      0x25, 0x01,
      0x75, 0x01,
      0x95, 0x08,
      0x81, 0x02,
      0x95, 0x01,
      0x75, 0x08,
      0x81, 0x01,
      0x95, 0x05,
      0x75, 0x01,
      0x05, 0x08,
      0x19, 0x01,
      0x29, 0x05,
      0x91, 0x02,             
      0x95, 0x01,             
      0x75, 0x03,             
      0x91, 0x01,
      0x95, 0x06, 		      
      0x75, 0x08,             
      0x15, 0x00,            
      0x25, 0x65,
      0x05, 0x07,
      0x19, 0x00,
      0x29, 0x65,
      0x81, 0x00,
      0xC0  
};


#define HID_MOUSE_REPORT_DESCRIPTOR_SIZE \
	sizeof (g_HID_au8MouseReportDescriptor) / sizeof(g_HID_au8MouseReportDescriptor[0])
const uint32_t g_HID_u32MouseReportDescriptorSize = HID_MOUSE_REPORT_DESCRIPTOR_SIZE;


#define HID_KEYBOARD_REPORT_DESCRIPTOR_SIZE \
    sizeof(g_HID_au8KeyboardReportDescriptor) / sizeof(g_HID_au8KeyboardReportDescriptor[0])
const uint32_t g_HID_u32KeyboardReportDescriptorSize = HID_KEYBOARD_REPORT_DESCRIPTOR_SIZE;


#if(HID_FUNCTION == HID_KEYBOARD)
# define HID_REPORT_DESCRIPTOR_SIZE HID_KEYBOARD_REPORT_DESCRIPTOR_SIZE
#else
# define HID_REPORT_DESCRIPTOR_SIZE HID_MOUSE_REPORT_DESCRIPTOR_SIZE
#endif

#if(HID_FUNCTION == HID_MOUSE)
__align(4) uint8_t g_au8MouseReport[3];
const uint32_t g_u32MouseReportSize = sizeof(g_au8MouseReport) / sizeof(g_au8MouseReport[0]);
#else
__align(4) uint8_t g_au8KeyboardReport[8];
const uint32_t g_u32KeyboardReportSize = sizeof(g_au8KeyboardReport) / sizeof(g_au8KeyboardReport[0]);
#endif

__align(4) const uint8_t g_HID_au8DeviceDescriptor[] =
{
	LEN_DEVICE,		// bLength
	DESC_DEVICE,	// bDescriptorType
	0x10, 0x01,		// bcdUSB
	0x00,			// bDeviceClass
	0x00,			// bDeviceSubClass
	0x00,			// bDeviceProtocol
	HID_MAX_PACKET_SIZE_EP0,	// bMaxPacketSize0
	// idVendor
	USB_VID & 0x00FF,
	(USB_VID & 0xFF00) >> 8,
	// idProduct
	USB_PID & 0x00FF,
	(USB_PID & 0xFF00) >> 8,
	0x00, 0x00,		// bcdDevice
	0x01,			// iManufacture
	0x02,			// iProduct
	0x03,			// iSerialNumber
	0x01			// bNumConfigurations
};

__align(4) const uint8_t g_HID_au8ConfigDescriptor[] =
{
	LEN_CONFIG,		// bLength
	DESC_CONFIG,	// bDescriptorType
	// wTotalLength
	LEN_CONFIG_AND_SUBORDINATE & 0x00FF,
	(LEN_CONFIG_AND_SUBORDINATE & 0xFF00) >> 8,
	0x01,			// bNumInterfaces
	0x01,			// bConfigurationValue
	0x00,			// iConfiguration
	(HID_IS_SELF_POWERED << 6) | (HID_IS_REMOTE_WAKEUP << 5),// bmAttributes
	HID_MAX_POWER,			// MaxPower

	// I/F descr: HID
	LEN_INTERFACE,	// bLength
	DESC_INTERFACE,	// bDescriptorType
	0x01,			// bInterfaceNumber
	0x00,			// bAlternateSetting
	0x01,			// bNumEndpoints
	0x03,			// bInterfaceClass
	0x01,			// bInterfaceSubClass
	HID_FUNCTION,			// bInterfaceProtocol
	0x00,			// iInterface

	// HID Descriptor
	LEN_HID,		// Size of this descriptor in UINT8s.
	DESC_HID,		// HID descriptor type.
	0x10, 0x01, 	// HID Class Spec. release number.
	0x00,			// H/W target country.
	0x01,			// Number of HID class descriptors to follow.
	DESC_HID_RPT,	// Dscriptor type.
	// Total length of report descriptor.
	HID_REPORT_DESCRIPTOR_SIZE & 0x00FF,
	(HID_REPORT_DESCRIPTOR_SIZE & 0xFF00) >> 8,

	// EP Descriptor: interrupt in.
	LEN_ENDPOINT,	// bLength
	DESC_ENDPOINT,	// bDescriptorType
	0x81,			// bEndpointAddress
	EP_INT,			// bmAttributes
	// wMaxPacketSize
	HID_MAX_PACKET_SIZE_EP1 & 0x00FF,
	(HID_MAX_PACKET_SIZE_EP1 & 0xFF00) >> 8,
	HID_DEFAULT_INT_IN_INTERVAL		// bInterval
};

__align(4) const uint8_t g_HID_au8StringLang[4] =
{
	4,				// bLength
	DESC_STRING,	// bDescriptorType
	0x09, 0x04
};

__align(4) const S_DRVUSB_STRING_DESC g_HID_sVendorStringDesc =
{
	16,
	DESC_STRING,
	{'N', 'U', 'V', 'O', 'T', 'O', 'N'}
};

__align(4) const S_DRVUSB_STRING_DESC g_HID_sProductStringDesc =
{
	16,
	DESC_STRING,
	{'W', 'P', 'M', ' ', 'U', 'S', 'B'}
};

__align(4) const uint8_t g_HID_au8StringSerial[26] =
{
	26,				// bLength
	DESC_STRING,	// bDescriptorType
	'B', 0, 
	'0', 0, 
	'2', 0, 
	'0', 0, 
	'0', 0, 
	'6', 0, 
	'0', 0, 
	'9', 0, 
	'2', 0, 
	'1', 0, 
	'1', 0, 
	'4', 0
};

void HID_Init(void *pfGetInReport, void *pfSetOutReport)
{
	/* Open HID to initial the descriptors and control handlers */
	HID_Open(HID_REPORT_SIZE, pfGetInReport, pfSetOutReport);
	
#if(HID_FUNCTION == HID_MOUSE)

    /* Set the HID report descriptor */
	HID_SetReportDescriptor(g_HID_au8MouseReportDescriptor, g_HID_u32MouseReportDescriptorSize);

    /* Set the HID report buffer */
	HID_SetReportBuf(g_au8MouseReport, g_u32MouseReportSize);

#else
    /* Set the HID report descriptor */
	HID_SetReportDescriptor(g_HID_au8KeyboardReportDescriptor, g_HID_u32KeyboardReportDescriptorSize);

    /* Set the HID report buffer */
	HID_SetReportBuf(g_au8KeyboardReport, g_u32KeyboardReportSize);
#endif

}

/* This callback will be executed whenever the interrupt IN transaction is received from HOST to get
   report. */
#if(HID_FUNCTION == HID_MOUSE)

void HID_GetInReport(uint8_t *buf)
{
#if 0
    int8_t ch;

    /* Use 'a', 'b', 'c', 'd' from UART to control right, left, up, and down */
    /* Use ',','.' to control button */
    if ( DrvUART_kbhit() )
    {
        ch = getchar();
        if(ch == 'a')
            buf[1] += 1;
        
        if(ch == 'b')
            buf[1] += -1;
            
        if(ch == 'c')
            buf[2] += 1;
        
        if(ch == 'd')
            buf[2] += -1;
       
        if(ch == ',')
            buf[0] |= 1;
        
        if(ch == '.')
            buf[0] |= 2;
                        
        //DrvSIO_printf("[0] = %x  [1] = %x  [2] = %x\n", buf[0], buf[1], buf[2]);     
    }
    else
    {
        buf[0] = 0;
    }
#else
	buf[0] = 0x00;
	buf[1] = 0x01;
	buf[2] = 0x00;
#endif
}

#else

void HID_GetInReport(uint8_t *buf)
{
    int32_t i;
    uint32_t key = 0xF;
	
	//key = inp32(GPIOA_PIN);
	{
	    /* No any key pressed */
	    if (((key >> 3) & 0xF) == 0xF)
	    {
		    for (i=0;i<8;i++)
		    {
		        buf[i] = 0;
		    }
	    }
	    
	    /* Key A (GPIOA3) Pressed */
	    if((key & (1<<3)) == 0)
	    {
	        buf[2] = 0x04;
	    }
	    
	    /* Key B (GPIOA4) Pressed */
	    if((key & (1<<4)) == 0)
	    {
	        buf[2] = 0x05;   
	    }
	    
	    /* Key C (GPIOA5) Pressed */
	    if((key & (1<<5)) == 0)
	    {
	        buf[2] = 0x06; 
	    }
	    
	    /* Key D (GPIOA6) Pressed */
	    if((key & (1<<6)) == 0)
	    {
	        buf[2] = 0x04;   
	        buf[3] = 0x05;   
	        buf[4] = 0x06;   
	    }
	}
}

#endif


/* This callback will be executed whenever the interrupt OUT transaction is received from HOST to set
   report. */
void HID_SetOutReport(uint8_t *buf)
{

}

int32_t HID_MainProcess(void)
{
    int32_t i32Ret = 0;
	E_DRVUSB_STATE eUsbState;
	    
    i32Ret = DrvUSB_Open(0);
    if(i32Ret != 0)
        return i32Ret;
        
	while(1)
	{
		/* Disable USB-related interrupts. */
		_DRVUSB_ENABLE_MISC_INT(0);

		/* Enable float-detection interrupt. */
		_DRVUSB_ENABLE_FLD_INT();

		// Wait for USB connected.
		while (1)
		{
			// Order here is significant.
			// Give a chance to handle remaining events before exiting this loop.
			eUsbState = DrvUSB_GetUsbState();

			DrvUSB_DispatchEvent();

			if ( eUsbState >= eDRVUSB_ATTACHED &&
			     eUsbState != eDRVUSB_SUSPENDED )
			{
				break;
			}
		}
		
		/* Start HID and install the callback functions to handle in/out report */
		HID_Init((void *)HID_GetInReport, (void *)HID_SetOutReport);

		// Enable USB-related interrupts.
		_DRVUSB_ENABLE_MISC_INT(IEF_WAKEUP | IEF_WAKEUPEN | IEF_FLD | IEF_USB | IEF_BUS);

		// Poll and handle USB events.
		while(1)
		{
			eUsbState = DrvUSB_GetUsbState();

			DrvUSB_DispatchEvent();

			if (eUsbState == eDRVUSB_DETACHED)
			{
				break;
			}
		}

		// Disable USB-related interrupts.
		_DRVUSB_ENABLE_MISC_INT(0);		
    }
}

